﻿using BA.API;
using BA.Core.Configuration.AdditionalAttributes;
using BA.Core.Configuration.Application.SubConfigurations;
using BA.Core.Configuration.FormBase.Controls;
using BA.Core.Configuration.FormBase.Renderers.Base;
using BA.Core.CustomAttributes;
using BA.Core.Models;
using BA.Core.Tools;
using DevExpress.Web;
using DevExpress.Web.Mvc;
using DevExpress.XtraReports.Web.Native.ClientControls;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Web;
using System.Web.UI.WebControls;

namespace BA.Core.Configuration.FormBase.Renderers
{
    /// <summary>
    /// Renderer for the new control
    /// </summary>
    [FormControlRenderer(typeof(ValueAndAliasControl))]
    public class ValueAndAliasDefaultRenderer : ControlDefaultReadEditBindableRendererBase
    {
        /// <inheritdoc/>
        public override void EditMode()
        {
            ValueAndAliasControl valueAndAliasControl = (ValueAndAliasControl)RenderParameters.Control;

            // Feldnamen definieren (Datenmodell)
            string modelFieldName = valueAndAliasControl.OrmFieldName;
            ViewState.ViewStateData.AddOrUpdate(BindKeys.ModelProperty, modelFieldName);

            // Feldnamen definieren für die Felder im Browser definieren
            string uiValueFieldName = modelFieldName + "_Value";
            ViewState.ViewStateData.Add("ValueField", uiValueFieldName);
            string uiAliasFieldName = modelFieldName + "_Alias";
            ViewState.ViewStateData.Add("AliasField", uiAliasFieldName);

            // zu verwendendes Trennzeichen
            char delimiter = valueAndAliasControl.Delimiter.ToCharArray()[0];
            ViewState.ViewStateData.Add("Delimiter", delimiter);

            // Inhalt des gespeicherten Feldes holen und für die Felder vorbereiten
            string fieldContent = (string)ConvertFromModelPropertyType(GetValueFromModel());
            string[] splt = fieldContent?.Split(delimiter) ?? (new string[] { "", "" });
            string fieldContentPart1 = splt?[0] ?? "";
            string fieldContentPart2 = splt?.Length > 1 ? splt[1] : "";

            // Erstes Feld erzeugen und als Objekt hinzufügen
            MVCxFormLayoutItem mvcxItem = CreateTextBoxItem(valueAndAliasControl, uiValueFieldName, RenderParameters.RenderingContext.FormModel, "Wert", fieldContentPart1);
            RenderParameters.MvcxControls.Add(mvcxItem);

            // Zweites Feld erzeugen und als Objekt hinzufügen
            mvcxItem = CreateTextBoxItem(valueAndAliasControl, uiAliasFieldName, RenderParameters.RenderingContext.FormModel, "Alias", fieldContentPart2);
            RenderParameters.MvcxControls.Add(mvcxItem);
        }

        /// <summary>
        /// Methode zum erstellen einer Textbox
        /// </summary>
        /// <param name="valueAndAliasControl"></param>
        /// <param name="fieldName"></param>
        /// <param name="formModel"></param>
        /// <param name="label"></param>
        /// <param name="presetValue"></param>
        /// <returns></returns>
        private MVCxFormLayoutItem CreateTextBoxItem(ValueAndAliasControl valueAndAliasControl,
                string fieldName, DevExFormModel formModel, string label, string presetValue)
        {
            MVCxFormLayoutItem mvcxItem = new MVCxFormLayoutItem();

            // Der Typ des neuen Controls => es ist eine "TextBox".
            mvcxItem.NestedExtensionType = FormLayoutNestedExtensionItemType.TextBox;

            // setzen verschiedener Einstellungen auf das MVCxItem, wie zB der Feldname, der im  
            // Webbrowser verwendet werden soll, die Beschriftung vor dem Feld, der Hilfetext, etc
            mvcxItem.Name = fieldName;
            mvcxItem.FieldName = fieldName;
            mvcxItem.ClientVisible = valueAndAliasControl.Visible;
            mvcxItem.VerticalAlign = FormLayoutVerticalAlign.Top;
            mvcxItem.Caption = label;
            mvcxItem.CaptionSettings.Location = Api.Enum.EnumValueConverter.GetLabelPosition(valueAndAliasControl.LabelPosition);
            mvcxItem.CaptionSettings.VerticalAlign = FormLayoutVerticalAlign.Top;

            // Weitere bestimmte Einstellungen sind im sog. "Settings"-Unterobjekt verfügbar.
            // Das betrifft zB die Editierbarkeit des Feldes, bestimmte Javascript-Events im Browser,
            // die ausgeführt werden sollen, Abmessungen des Feldes, usw.
            TextBoxSettings settings = (TextBoxSettings)mvcxItem.NestedExtensionSettings;

            // Horizontale Ausrichtung des Controls
            settings.ControlStyle.HorizontalAlign = HorizontalAlign.Left;

            // Ist das Feld eventuell für Bearbeitung gesperrt?
            if (!IsControlEditable())
            {
                settings.ClientEnabled = false;
                ViewState.ShouldBind = false;
            }

            // Breite des Feldes
            settings.Width = new Unit(valueAndAliasControl.Width, Api.Enum.EnumValueConverter.GetValueWidthType(valueAndAliasControl.WidthType));

            // Feld kann als fehlerhaft markiert werden, falls die Validierung fehlschlägt
            if (settings.Properties is EditProperties)
            {
                EditProperties props = (EditProperties)settings.Properties;
                ApplicationSubConfigurationForm appFormSettings = Api.Config.ApplicationConfiguration().GetSubConfiguration<ApplicationSubConfigurationForm>();

                props.ValidationSettings.Display = Api.Enum.EnumValueConverter.GetErrorFrameDisplayMode(appFormSettings.ErrorFrameDisplayMode);
                props.ValidationSettings.ErrorTextPosition = Api.Enum.EnumValueConverter.GetErrorTextPosition(appFormSettings.ErrorTextPosition);
                props.ValidationSettings.ErrorDisplayMode = Api.Enum.EnumValueConverter.GetErrorDisplayMode(appFormSettings.ErrorDisplayMode);
                settings.ShowModelErrors = true;
            }

            // Der "placeholder", also der Text, der in einem leeren Feld angezeigt werden soll
            settings.Properties.NullText = Api.Text.Format(valueAndAliasControl.NullText);

            // vorbelegen des übergebenen im Datenmodell schon verfügbaren Wertes
            if (!string.IsNullOrWhiteSpace(presetValue))
                settings.PreRender = (sender, arg) =>
                {
                    MVCxTextBox txt = sender as MVCxTextBox;
                    txt.Value = presetValue;
                    txt.DataBind();
                };

            // spezieller Fix für FireFox, damit Text in Feldern selektiert und kopiert werden  
            // können, die keine Eingabe erlauben
            RenderingUtils.FixItemForFirefox(mvcxItem, settings, valueAndAliasControl.CssClass);
            return mvcxItem;
        }

        /// <inheritdoc/>
        public override void ReadMode()
        {
            string namePrefix = RenderingUtils.GetUIReadyContextPrefix(RenderParameters.RenderingContext);
            ValueAndAliasControl valueAndAliasControl = (ValueAndAliasControl)RenderParameters.Control;

            // Feldnamen setzen
            string modelFieldName = valueAndAliasControl.OrmFieldName;

            // Alternativ kann ein MVCxFormLayoutItem auf diese Weise erstellt werden.
            RenderParameters.MvcxControls.Add(item =>
            {
                // Typ des Items definieren und Settings casten
                item.NestedExtensionType = FormLayoutNestedExtensionItemType.Label;
                LabelSettings labelSettings = (LabelSettings)item.NestedExtensionSettings;

                // Wert aus dem Datensatz auslesen und in als Inhalt festlegen
                char delimiter = valueAndAliasControl.Delimiter.ToCharArray()[0];
                string fieldContent = (string)ConvertFromModelPropertyType(GetValueFromModel());
                fieldContent = fieldContent?.Split(delimiter)?[0] ?? "";
                labelSettings.Text = fieldContent;

                // Weieter Einstellungen festlegen
                labelSettings.Name = namePrefix + valueAndAliasControl.ControlInternalName + RenderParameters.MvcxControls.Count;
                labelSettings.Width = new Unit(valueAndAliasControl.Width, Api.Enum.EnumValueConverter.GetValueWidthType(valueAndAliasControl.WidthType));
                labelSettings.ControlStyle.CssClass = valueAndAliasControl.CssClass;
                item.HorizontalAlign = FormLayoutHorizontalAlign.Left;
                item.Caption = valueAndAliasControl.Caption?.Translate() ?? string.Empty;
                item.CaptionSettings.Location = Api.Enum.EnumValueConverter.GetLabelPosition(valueAndAliasControl.LabelPosition);
                SetHelpText(item);
                item.Visible = valueAndAliasControl.Visible;

                // Label setzen, falls konfiguriert
                if (string.IsNullOrWhiteSpace(item.Caption))
                    item.ShowCaption = DevExpress.Utils.DefaultBoolean.False;
            });
        }

        /// <inheritdoc/>
        public override void Bind()
        {
            // Auslesen der Daten von dem Value
            string bindValueFieldName_Value = (string)ViewState.ViewStateData["ValueField"];
            string value = (string)ConvertToModelPropertyType(GetValueFromRequest(bindValueFieldName_Value) ?? "");

            // Auslesen der Daten von dem Alias
            string bindValueFieldName_Alias = (string)ViewState.ViewStateData["AliasField"];
            string alias = (string)ConvertToModelPropertyType(GetValueFromRequest(bindValueFieldName_Alias) ?? "");

            // Berechnung des Wertes für das Model
            string delimiter = (string)ViewState.ViewStateData["Delimiter"];
            string modelValue = value + delimiter + alias;

            // Schreiben des Wertes in das Model
            WriteValueToModel(modelValue);
        }
    }
}