declare module BA.Ui.Actions {
    /**
     * Class used for creating actions for the SelectionBoundFloatingContextMenu
     */
    class SelectionBoundFloatingContextMenuItem {
        /**
         * ID of the node of the rendered action
         */
        private _HtmlElementId;
        /**
         * Timeout of the settings container
         */
        private _SettingsTimeout;
        /**
         * Caption of the action
         */
        Caption: string;
        /**
         * Help Text of the action
         */
        HelpText: string;
        /**
         * Name of the action (action id)
         */
        ActionName: string;
        /**
         * Name of the icon of  the action
         */
        IconName: string;
        /**
         * Indicator if the caption should be shown, default = true
         */
        ShowCaption: boolean;
        /**
         * Sort order of the rendered element
         */
        SortOrder: number;
        /**
         * Indicator if the action should be enabled or not
         */
        Enabled: boolean;
        /**
         * Indicator if the action should be visible or not
         */
        Visible: boolean;
        /**
         * Method the be performed on click of an element (can be null)
         */
        OnClick: (element: HTMLElement, event: MouseEvent) => void | null;
        /**
         * Method to perform additional actions on mouseenter, can be null
         */
        OnMouseEnter: (actionElement: HTMLElement) => void | null;
        /**
         * Method to perform additional actions on mouseenter, can be null
         */
        OnSettingsContainerShown: (actionElement: HTMLElement, settingsContainer: HTMLElement) => void | null;
        /**
         * Method to perform additional actions when the Context Menu is shown or updated
         */
        OnContextMenuUpdated: () => void | null;
        /**
        /**
         * Function to dynamically change the visibilityof an action
         */
        GetVisible: (element: HTMLElement) => boolean | null;
        /**
         * Method to Render child controls inside the Settings container (can be null)
         */
        RenderChildren: (actionElement: HTMLElement, settingsContainer: HTMLElement) => void | null;
        /**
         * Method to render the action item
         * @param menu the corresponding menu container to render the action in
         * @returns the HTML Element of the rendered action
         */
        Render(menu: SelectionBoundFloatingContextMenu): HTMLElement;
        /**
         * Creates a container for rendering settings
         * @param menu the corresponding menu container
         * @returns The settings container's HTML element
         */
        private _CreateSettingsContainer;
        /**
         * Get JQuery of the settings container
         * @returns JQuery object of the settings container
         */
        GetSettingsContainerJQ(): JQuery;
        /**
         * Update an action's visibility
         */
        UpdateVisibility(): void;
        /**
         * Update an action's editability
         */
        UpdateEditable(): void;
        /**
         * Show the settings container if it exists
         */
        ShowSettingsContainer(): void;
        /**
         * Hide the Settings container if it exists
         * @param immediate indicator if the hiding should be performed immediately or not
         */
        HideSettingsPanel(immediate?: boolean): void;
        /**
         * Initializes a new instance of the SelectionBoundFloatingContextMenuItem class.
         * @param name name of the action (the action's identifier)
         * @param caption Caption of the action
         * @param icon Icon of the Action
         * @param sortOrder Sort Order of the action (optional)
         */
        constructor(name: string, caption: string, helpText: string, icon: string, sortOrder: number | null);
    }
    /**
     * Class Used for managing inline context actions
     */
    class SelectionBoundFloatingContextMenu {
        /**
         * suffix
         */
        readonly _Suffix: string;
        LastSelectionRange: Range;
        LockUpdate: boolean;
        /**
         * Actions configured within the context menu
         */
        private Actions;
        /**
         * Name of the owner control
         */
        OwnerControlName: string;
        /**
         * HTML Element of the Menu container
         */
        MenuWrapper: HTMLElement;
        /**
         * Client communication Model used
         */
        ClientCommunicationModel: BA.Ui.Models.OpenAIClientCommunicationModel;
        /**
         * Indicator if the selection was changed by an action from this context menu
         */
        SelectionChangedByAction: boolean;
        /**
         * Function to update the selection range (individual implementation for each control)
         */
        UpdateSelectionRange: () => Range;
        /**
         * Function to dynamically determine the minimum top position
         */
        GetMinTopPosition: () => number | null;
        /**
         * Initializes the context menu
         * @param ownerControl the corresponding Control
         */
        Init(ownerControl: ASPxClientControl): void;
        /**
         * Function to be called for rendering and adding an action to the context menu
         * @param action
         * @returns The action's HTML Element
         */
        RenderAction(action: SelectionBoundFloatingContextMenuItem): HTMLElement;
        /**
         * Function to update the actions visibilities
         */
        UpdateActions(): void;
        /**
         * Function to show the context menu
         */
        Show(): void;
        /**
         * Function to position the Context menu above the selection
         */
        PositionContainer(): boolean;
        /**
         * Function to Hide the Context Menu
         */
        Hide(): void;
        /**
         * Function to handle a changed selection
         * @param selectedText The text content of the selection
         * @param deleteHistory indicator wether history should be deleted or not, default = false
         */
        HandleSelectionChanged(selectedText: string, deleteHistory?: boolean): void;
        /**
         * Function to handle changes and update the menu
         */
        HandleChanges(): void;
        /**
         * Helper function to create a slider element
         * @param wrapperElement the wrapper container to use
         * @param minValue the minimum value of the slider
         * @param maxValue the maximum value of the slider
         * @param defaultValue the default value of the slider
         * @param stepSize the step size of the slider
         * @param id a custom id for the element
         * @returns The HTML range input element which is already appended to the wrapper
         */
        CreateSliderElement(wrapperElement: HTMLElement, minValue: number, maxValue: number, defaultValue: number, stepSize?: number | null, id?: string): HTMLInputElement;
        /**
         * Helper function to create a text element
         * @param wrapperElement The wrapper container to use
         * @param innerText The text to be displayed
         * @param className optional className to add
         * @returns The HTML element which is already appended to the wrapper
         */
        CreateTextElement(wrapperElement: HTMLElement, innerText: string, className?: string): HTMLElement;
        /**
         * Get the currently used ClientCommunicationModel (or create a new one if there is none yet)
         * @returns the current clientCommunicationModel
         */
        GetCurrentClientCommunicationModel(): BA.Ui.Models.OpenAIClientCommunicationModel;
        /**
         * Initializes a new instance of the SelectionBoundFloatingContextMenu
         */
        constructor();
    }
}
